with System;
with NXT.AVR;		  use NXT.AVR;
with Nxt.Display;         use Nxt.Display;
with NXT.Motor_Encoders;
with NXT.Battery;
with NXT.Buttons; use NXT.Buttons;

package body Tasks is

   ---------------------------
   --  Auxiliary Functions  --
   ---------------------------


   function Get_Execution_Time return Integer is
   begin
      return Integer (To_Duration (Clock - Time_Zero));
   end Get_Execution_Time;
   --  This function calculates Real_Time.Clock - Time_Zero



   procedure Stop is
   begin
      --  Stops the right motor.
      Control_Motor (Right_Motor_Id, Speed_Stop, Brake);
      --  Stops the left motor.
      Control_Motor (Left_Motor_Id, Speed_Stop, Brake);
   end Stop;





   -----------------------
   --  Background task  --
   -----------------------
   procedure Background is
   begin
      loop
         null;
      end loop;
   end Background;

   -------------
   --  Tasks  --
   -------------
   task M_A is
      pragma Priority (System.Priority'First + 2);
      pragma Storage_Size (4096);
   end M_A;

   task M_B is
      pragma Priority (System.Priority'First + 2);
      pragma Storage_Size (4096);
   end M_B;

   task Display is
      pragma Priority (System.Priority'First + 1);
      pragma Storage_Size (4096);
   end Display;


   task body M_A is
      Right : Boolean := False;
      Degrees : Integer := 0;
      Next_Time : Time := Time_Zero;
   begin
      Stop;
      loop
         delay until Next_Time;

         Right := NXT.Touch_Sensors.Pressed (Touch_1);
         if Right then
            Control_Motor (Right_Motor_Id, Speed_Full, Backward);
         else
            Control_Motor (Right_Motor_Id, Speed_Stop, Brake);
         end if;
         --	 if Button = Power_Button then
         if Current_Button = Power_Button then
            Stop;
            Power_Down;
         end if;
         Next_Time := Next_Time + Period_Movement;
      end loop;
   end M_A;

   task body M_B is
      Left  : Boolean := False;
      Degrees : Integer := 0;
      Next_Time : Time := Time_Zero;
   begin
      Stop;
      loop
         delay until Next_Time;
         Left := NXT.Touch_Sensors.Pressed (Touch_2);
         if Left then
            Control_Motor (Left_Motor_Id, Speed_Full, Backward);
         else
            Control_Motor (Left_Motor_Id, Speed_Stop, Brake);
         end if;

         if Current_Button = Power_Button then
            Stop;
            Power_Down;
         end if;
         Next_Time := Next_Time + Period_Movement;
      end loop;
   end M_B;


   task body Display is
      Next_Time : Time := Time_Zero;
   begin
      loop
         delay until Next_Time;
         Clear_Screen_Noupdate;
         Set_Pos (0,0);
         Put_Noupdate ("   vehicle_v12");
         Newline_Noupdate;
         Put_Noupdate ("    running...");
         Newline_Noupdate;
         Newline_Noupdate;
         Put_Noupdate ("time: ");
         Put_Noupdate (Get_Execution_Time);
         Put_Noupdate (" s");
         Newline_Noupdate;
         Put_Noupdate ("battery: ");
         Put_Noupdate (Integer (Battery.Millivolts));
         Put_Noupdate (" mV");
         Newline_Noupdate;
         Put_Noupdate ("      EXIT");
         Screen_Update;
         Next_Time := Next_Time + Period_Display;
      end loop;
   end Display;

end Tasks;
